"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.onRouteLoaded = onRouteLoaded;
exports.default = void 0;

var _propTypes = _interopRequireDefault(require("prop-types"));

var _react = _interopRequireWildcard(require("react"));

var _reactRouter = require("react-router");

var _Switch = require("./Switch");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const routeLoadedHandlers = [];

function onRouteLoaded(handleRouteLoaded) {
  const index = routeLoadedHandlers.length;
  routeLoadedHandlers.push(handleRouteLoaded);
  return {
    remove: () => delete routeLoadedHandlers[index]
  };
}

class Route extends _react.Component {
  constructor(...args) {
    super(...args);

    _defineProperty(this, "componentDefer", null);

    _defineProperty(this, "renderDefer", null);

    _defineProperty(this, "state", {
      component: () => null,
      isLoading: false,
      rendered: null
    });

    _defineProperty(this, "isLoading", true);

    _defineProperty(this, "initialRouteLoaded", false);

    _defineProperty(this, "previousHistoryLocation", null);
  }

  getRenderDefer() {
    if (this.isLoading && !this.state.isLoading) this.triggerRenderDefer();
    const rendered = this.state.rendered;
    return () => rendered;
  }

  getComponentDefer() {
    if (this.isLoading && !this.state.isLoading) this.triggerComponentDefer();
    const component = this.state.component;
    return component;
  }

  triggerComponentDefer() {
    var _this = this;

    return _asyncToGenerator(function* () {
      _this.startedLoading();

      let component = yield _this.componentDefer(_this.props);
      if (component.__esModule) component = component.default;

      _this.finishedLoading({
        component
      });
    })();
  }

  triggerRenderDefer() {
    var _this2 = this;

    return _asyncToGenerator(function* () {
      _this2.startedLoading();

      let rendered = yield _this2.renderDefer(_this2.props);
      if (rendered.__esModule) rendered = rendered.default;

      _this2.finishedLoading({
        rendered
      });
    })();
  }

  startedLoading() {
    if (this.componentDefer || this.renderDefer) {
      this.state.isLoading = true;
      setTimeout(() => this.forceUpdate(), 0);
    }

    const router = this.context.router;
    this.onLoadingStart(router);
  }

  finishedLoading({
    rendered,
    component
  }) {
    var _this3 = this;

    return _asyncToGenerator(function* () {
      const router = _this3.context.router;
      if (!_this3.initialRouteLoaded) _this3.initialRouteLoaded = true;

      _this3.onLoadingFinish(router);

      for (var _i = 0; _i < routeLoadedHandlers.length; _i++) {
        const handleRouteLoaded = routeLoadedHandlers[_i];

        if (typeof handleRouteLoaded === 'function') {
          yield handleRouteLoaded(router);
        }
      }

      _this3.isLoading = false;

      if (_this3.componentDefer || _this3.renderDefer) {
        _this3.setState(_objectSpread({
          isLoading: false
        }, rendered ? {
          rendered
        } : {}, component ? {
          component
        } : {}));
      }
    })();
  }

  renderContext(context) {
    const isLoading = this.state.isLoading;
    const onLoadingStart = context.onLoadingStart,
          onLoadingFinish = context.onLoadingFinish;

    if (!this.onLoadingStart) {
      this.onLoadingStart = this.props.onLoadingStart || onLoadingStart;
    }

    if (!this.onLoadingFinish) {
      this.onLoadingFinish = this.props.onLoadingFinish || onLoadingFinish;
    }

    const _this$props = this.props,
          component = _this$props.component,
          render = _this$props.render,
          componentDefer = _this$props.componentDefer,
          renderDefer = _this$props.renderDefer;

    if (component) {
      if (isPromise(component)) {
        this.componentDefer = component;
      }
    } else if (render) {
      if (isPromise(render)) {
        this.renderDefer = render;
      }
    } else if (componentDefer) {
      this.componentDefer = componentDefer;
    } else if (renderDefer) {
      this.renderDefer = renderDefer;
    }

    if (!this.componentDefer && !this.renderDefer) this.startedLoading();

    let props = _objectSpread({}, this.props);

    delete props.componentDefer;
    delete props.renderDefer;

    if (this.componentDefer) {
      props = _objectSpread({}, props, {
        component: this.getComponentDefer()
      });
    } else if (this.renderDefer) {
      props = _objectSpread({}, props, {
        render: this.getRenderDefer()
      });
    } else {
      this.finishedLoading({});
    }

    if (isLoading) this.renderLoading(context, props);
    return _react.default.createElement(_reactRouter.Route, props);
  }

  renderLoading({
    loading,
    loadingAll
  }, props) {
    if (!this.initialRouteLoaded || this.props.loadingAll || loadingAll) {
      delete props.component;
      delete props.render;

      props.component = () => this.props.loading || loading;
    }

    return _react.default.createElement(_reactRouter.Route, props);
  }

  render() {
    const location = this.context.router.history.location;
    if (location !== this.previousHistoryLocation) this.isLoading = true;
    this.previousHistoryLocation = location;
    return _react.default.createElement(_Switch.SwitchContext.Consumer, null, context => this.renderContext(context));
  }

}

exports.default = Route;

_defineProperty(Route, "propTypes", {
  componentDefer: _propTypes.default.func,
  loading: _propTypes.default.node,
  loadingAll: _propTypes.default.bool,
  onLoadingFinish: _propTypes.default.func,
  onLoadingStart: _propTypes.default.func,
  renderDefer: _propTypes.default.func
});

_defineProperty(Route, "defaultProps", {
  componentDefer: null,
  loading: null,
  loadingAll: false,
  onLoadingFinish: null,
  onLoadingStart: null,
  renderDefer: null
});

_defineProperty(Route, "contextTypes", {
  router: _propTypes.default.shape({
    history: _propTypes.default.shape({
      push: _propTypes.default.func.isRequired
    }).isRequired
  }).isRequired
});

function isPromise(promise) {
  const _ref = (promise === null || promise === void 0 ? void 0 : promise.constructor) || {},
        name = _ref.name;

  if (name === 'Promise' || name === 'LazyPromise') return true;

  if (typeof promise === 'function') {
    try {
      var _promise;

      const _ref2 = ((_promise = promise()) === null || _promise === void 0 ? void 0 : _promise.constructor) || {},
            name = _ref2.name;

      if (name === 'Promise' || name === 'LazyPromise') return true;
    } catch (err) {}
  }

  return false;
}